//---------------------------------------------------------------------------

#ifndef smcH
#define smcH
//---------------------------------------------------------------------------
#include <classes.hpp> // TMemoryStream
#include <graphics.hpp> // TColor
#include <vector>

#define COUNT(a) (sizeof(a)/sizeof((a)[0]))

#pragma pack(push,1) // byte align
typedef struct{
   unsigned char bg[24][3][10];
   unsigned char fg[24][3][10];
   unsigned char md[24][3][10];
   unsigned char block_gg[256];
   unsigned char block_fg[256];
   unsigned char block_ob[256];
   unsigned char block_fl[256];
   unsigned char rm_links[24][4];
   unsigned char start_rm,start_ps,start_dr,zero;
   unsigned char guard_ps[24];
   unsigned char guard_tp[24];
   unsigned char guard_sk[24];
   unsigned char guard_uk1[24];
   unsigned char guard_uk2[24];
   unsigned char guard_uk3[24];
   unsigned char door1[256];
   unsigned char door2[256];
}TLevel;
#if sizeof(TLevel)!=3940
#error Level size incorrect!
#endif

// For copy/paste
typedef struct{
   unsigned char bg[3][10];
   unsigned char fg[3][10];
   unsigned char md[3][10];
   //unsigned char rm_links[4];
   unsigned char guard_ps;
   unsigned char guard_tp;
   unsigned char guard_sk;
   unsigned char guard_uk1;
   unsigned char guard_uk2;
   unsigned char guard_uk3;
}TRoom;
#pragma pack(pop)

class TSMC {
   private:
   TMemoryStream* ms_smc;
   TMemoryStream** ms_data; int ndata;
   int SMCHD;
   bool gr_edited[3];
   TMemoryStream* bg_rest;
   TMemoryStream* fg_rest;
   TMemoryStream* gg_rest;
   //Graphics::TBitmap* bm_torch;
   public:
   int curr_level,curr_rm;
   int ltype,otype;
   bool want_label;
   unsigned __int16* bgblock;
   unsigned char* bgtile;
   unsigned __int16* fgblock;
   unsigned char* fgtile;
   unsigned __int16* ggblock;
   unsigned char* ggtile;
   TColor* pal,*pal2;
   bool sel[3][10];
   TLevel level;
   TSMC();
   ~TSMC();
   bool opensmc(AnsiString);
   void clear();
   void openlevel();
   void decodelevel(TMemoryStream*,TMemoryStream*,int);
   void openenv(int);
   void decodeenv(TMemoryStream* ms_in,unsigned __int16**block,unsigned char**tile,int wh,int hh,int si,int nb,int nt,int Bib,int Bit,TMemoryStream*&rest);
   void decodearray(TMemoryStream*,int,int,unsigned __int16*);
   void decodeblock(TMemoryStream* ms,int n,int si,unsigned __int16* ar,bool big);
   void decodetile16(TMemoryStream*,int,unsigned char*);
   void decodetile32(TMemoryStream*,int,unsigned char*);
   void drawroom(Graphics::TBitmap*);
   void drawlayer(Graphics::TBitmap*,unsigned char[3][10],int,int,int,unsigned __int16*,unsigned char*,unsigned char*,unsigned char,unsigned char*);
   void drawblock(Graphics::TBitmap*,int,int,int,int,int,int,unsigned __int16*,unsigned char*,unsigned char,bool);
   void drawtile(Graphics::TBitmap*,int,int,int,int,unsigned char*,unsigned char,bool);
   void decodepal(TMemoryStream*,TColor**,int);
   void brightpal(TColor*,TColor*/***/,int);
   void clearsel();
   void savelevel();
   void savesmc(AnsiString);
   void encodelevel(TMemoryStream*,TMemoryStream*,int);
//   unsigned __int32 lastsa;
   void decodesprite(TMemoryStream*,unsigned char**,unsigned char*,unsigned char*,unsigned char*,unsigned char*);
   void drawsprite(Graphics::TBitmap*,int,int,int,int,bool,bool);
   void decodeobj(TMemoryStream* ms_in,unsigned char**tile);
   void drawobj(Graphics::TBitmap* bm,int x0,int y0,int ix,bool sel);
   void drawanobj(Graphics::TBitmap* bm,int x0,int y0,unsigned char ch,bool sel);
   void drawtorch(Graphics::TBitmap* bm,int x0,int y0,bool sel);
   void closeenv();
   void* getaddr(int addr);
   void set_ltype(int level,int lt);
   void editenvpal();
   void editpal(int index,AnsiString pt);
   void encodepal(TMemoryStream*,TColor*,int);
   void editspritespal();
   void editlevelgr(int layer);

   void encodeenv(TMemoryStream* ms_out,unsigned __int16**block,unsigned char**tile,int wh,int hh,int si,int nb,int nt,int Bib,int Bit,TMemoryStream* rest);
   void encodeblocks(TMemoryStream*,int,int,unsigned __int16*,bool big);
   void encodeblock(TMemoryStream*,int,unsigned __int16*,bool big);
   void encodetile16(TMemoryStream*,int,unsigned char*);
   void encodetile32(TMemoryStream*,int,unsigned char*);
   void saveenv();
   void determine_array(int n,int si,char* items,TMemoryStream*& ms_new_items,std::vector<unsigned __int16> & out_mapping,int bits);

   int get_block_fl(int ix);
   void set_block_fl(int ix,int value);
   int get_bganim(int ix);
   void set_bganim(int ix,int value);

   void swap_room_positions(int room1, int room2);
   void swap_room_details(int room1, int room2);
   void swap_room_transparent(int room1, int room2);
   bool want_floors_walls;
};
extern const char* ob[0x2C];
extern const char* gu[0x19];
extern const char* levels[0x1B];
extern const char* ltypes[10];

#endif

